//
//  NZB.h
//  UsenetEngine
//
//  Created by Ger Teunis on 12/6/09.
//  Copyright (c) 2013 VortexApps. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "NNTPEnums.h"
@class File;
@class FileArticle;
@class ConnectionPoolItem;
@class PersistenceHandler;

typedef enum {
    ExtractPasswordsRequiredStateNo,
    ExtractPasswordsRequiredStateYes,
    ExtractPasswordsRequiredStateAllEntered
} ExtractPasswordsRequiredState;

@interface NZB : NSObject {
	
    int databaseId;
	NSString *uiTitle;
	NSString *nzbFileName;
	NSMutableArray *_fileQueue;
	bool isPaused;
	NZBState _state;
	NSString *statusText;
	NSString *destinationPath;
	int transferedBytes;
	int transferedSpeed;
	float progress;
	long long downloadedSize;
	long long totalDownloadSize;
	NSLock *downloadSizeLock;
	NSLock *unsavedArticleBytesLock;
	bool usedSmartPar2Skip;
	NSTask *postProcessingTask;
	unsigned long long unsavedArticleBytes;
    NSString *cachedNZBRepresentingFileName;
    bool cachedNZBRepresentingFileNameIsDir;
    NSString *nzbHash;
    bool usedPauseOnPasswordProtectedNZB;
    bool hasArticlesForBackupServer;
    NSDate *postDate;
    NSLock *transferredSizeLock;
    bool cachedHasPreviewableVideoFile;
    NSLock *cachedHasPreviewableVideoFileLock;
    NSUInteger totalArticleCount;
    NSUInteger failedArticleCount;
    bool usedPauseOnTooManyFailedArticles;
    NSLock *articleCountLock;
    NSString *groupUUID;
    NSString *mergeAddCorrelationUUID;
    NSMutableArray *transferredSpeedHistory;
    NSNumber *mightContainFrenchRenameSet;
    NSNumber *cachedHasPAR2Files;
    bool usedPauseOnSFVChecksumFailed;
    NSString *addedViaRSSWatchName;
    bool invalidFileSubjectsInNZB;
    NSDictionary *addNZBHTTPHeaders;
    bool persistenceShouldReportChanges;
    PersistenceHandler *persistenceHandler;
    NSNumber *par2ReportsFileIssues;
}

@property (nonatomic) int databaseId;
@property (nonatomic, retain) NSString *uiTitle;
@property (nonatomic, retain) NSString *nzbFileName;
@property (nonatomic) bool isPaused;
@property (nonatomic) NZBState state;
@property (nonatomic, retain) NSString *statusText;
@property (nonatomic, retain) NSString *destinationPath;
@property (nonatomic) int transferedSpeed;
@property (nonatomic) float progress;
@property (nonatomic) long long downloadedSize;
@property (nonatomic) long long totalDownloadSize;
@property (nonatomic, retain) NSLock *downloadSizeLock;
@property (nonatomic, retain) NSLock *unsavedArticleBytesLock;
@property (nonatomic) bool usedSmartPar2Skip;
@property (nonatomic, retain) NSTask *postProcessingTask;
@property (nonatomic, retain) NSString *nzbHash;
@property (nonatomic) bool usedPauseOnPasswordProtectedNZB;
@property (nonatomic) bool hasArticlesForBackupServer;
@property (nonatomic, retain) NSDate *postDate;
@property (nonatomic) bool cachedHasPreviewableVideoFile;
@property (nonatomic) NSUInteger totalArticleCount;
@property (nonatomic) NSUInteger failedArticleCount;
@property (nonatomic) bool usedPauseOnTooManyFailedArticles;
@property (nonatomic, retain) NSString *groupUUID;
@property (nonatomic, retain) NSString *mergeAddCorrelationUUID;
@property (nonatomic, retain) NSNumber *cachedHasPAR2Files;
@property (nonatomic) bool usedPauseOnSFVChecksumFailed;
@property (nonatomic, retain) NSString *addedViaRSSWatchName;
@property (nonatomic, assign) bool invalidFileSubjectsInNZB;
@property (nonatomic, retain) NSDictionary *addNZBHTTPHeaders;
@property (nonatomic, assign) bool persistenceShouldReportChanges;
@property (nonatomic, retain, readonly) NSNumber *par2ReportsFileIssues;

#pragma mark -
#pragma mark Sort file list
-(void)sortStartFilesToTop:(NSArray*)files inFileQueue:(NSMutableArray*)FileQueue;
-(int)quickSortPartition:(NSMutableArray*)array leftIndex:(int)left rightIndex:(int)right;
-(void)quickSort:(NSMutableArray*)array leftIndex:(int)left rightIndex:(int)right;
-(void)sortFileList:(NSMutableArray*)fileList withStartFilesOnTop:(bool)startFilesOnTop;
-(NSArray*)getSFVFiles:(NSArray*)fileList;

#pragma mark -
#pragma mark Sizes and states
-(bool)mightContainFrenchRenameSet;
-(bool)hasObfuscatedFilenames;
-(BOOL)stringLikelyObfuscated:(NSString*)fileName includingFileExtension:(BOOL)includeExtension;
-(void)updateHasArticlesForBackupServer;
-(void)detectJunkDownloads;
-(void)detectBeforeDownloadWarnings;
-(void)calculateSizes;
-(void)calculateTotalDownloadSize;
-(void)calculateDownloadedSize;
-(void)addDownloadedSize:(int)downloaded;
-(unsigned long long)getUnsavedBytes;
-(void)addTransferredBytes:(int)bytes;
-(int)calculateTransferSpeed;
-(bool)isSmartSkippable;
-(void)addUnsavedArticleBytes:(int)unsavedBytes;
-(void)removeUnsavedArticleBytes:(int)savedBytes;
-(void)pause:(NSString*)reason;
-(void)resume;
-(void)cancelDownload:(bool)removeFiles;
-(ExtractPasswordsRequiredState)extractPasswordRequiredState;
-(bool)retryDownloadMissingParts;
-(void)updatePostDate;
-(void)checkBackupServerArticlesValidity;

#pragma mark -
#pragma mark Skip file methods
-(void)skipParFiles;
-(void)unskipParFiles;
-(void)skipSampleFiles;
-(void)unskipSampleFiles;
-(NSArray*)getSkippableSampleFiles;
-(NSArray*)getSkippableParFiles;
-(void)downloadAllPAR2Files;
-(void)moveFilesFromNZB:(NZB*)nzb par2Only:(BOOL)par2Only;
-(void)removeNonPar2FilesFromQueue;

#pragma mark -
#pragma mark Fix old state numbers
-(void)continueDownloadFixStates;

#pragma mark -
#pragma mark Rest
-(BOOL)reportFailedToNewznabDidAddNewDownload;
-(BOOL)unskipNonPar2Files;
-(BOOL)userSkippedFiles;
-(BOOL)skipFileSet:(NSArray*)files skip:(BOOL)skip testOnly:(BOOL)testOnly;
-(BOOL)containsTooSmallStartArchive;
-(BOOL)containsMissingStartArchive;
-(bool)addFiles:(NSMutableArray*)files;
-(void)dumpFileQueue;
-(void)deleteCompletedFiles;
-(bool)nzbHasTooManyFailedArticles;
-(void)addFailedArticle;
-(void)updateCachedHasPreviewableVideoFile;
-(void)removeFile:(File*)file;
-(void)addFileToSmartCheckStateFile:(File*)file;
-(bool)fileInSmartCheckStateFile:(File*)file;
-(File*)findPlayablePreviewFile;
-(NSMutableArray*)fileQueue;
-(void)setFileQueue:(NSMutableArray*)files;
-(bool)fileQueueLoaded;
-(NSString*)getNZBRepresentingFileName:(BOOL*)isDir;
-(bool)isCandidateForRecursiveUncompress;
-(void)replaceFilesWithFolderContents:(NSString*)folder;
-(void)createDestinationPath;
-(void)pausePostProcessing;
-(void)resumePostProcessing;
-(void)stopPostProcessing;
-(bool)closeAllFileHandles;
-(int)downloadArticle:(ConnectionPoolItem*)connection isFinalResult:(BOOL*)finalResult;
-(long long)saveFiles:(long long)maxSaveSize;
-initWithFilename:(NSString*)nzbFilename DatabaseId:(int)DatabaseId DestinationPath:(NSString*)Path;
-(NSArray*)startArchives;
-(void)dealloc;

@end
